/*
 * Copyright (c) 2004, Jeremy Rayner
 * All rights reserved.
 */
package org.javanicus.easy;


/** a simplification of the instantiation management facilities of lightweight containers.
 * <p>
 * This is an example of the usage:<br>
 * <code>
 * easy.reg(Boy.class);<br>
 * easy.reg(Girl.class);<br>
 * girl = (Girl)easy.get(Girl.class);<br>
 * </code>
 * <p>
 * or in groovy<br>
 * 
 * <code>
 * [Boy.class, Girl.class].each{easy.reg(it)}<br>
 * girl = easy.get(Girl.class)<br>
 * </code>
 * <p>
 * All of these methods may throw RuntimeExceptions specific to the container underneath.
 */
public interface EasyContainer {
    
    /**
     * Gets a component instance.
     * 
     * @param key key the component was registered with
     * 
     * @return an instantiated component
     */
    public Object get(Object key);
    
    /**
     * Registers a component using componentImplementation as key.
     * 
     * @param implementation the concrete component class
     * 
     * @return the key used for registration
     */
    public Object reg(Class implementation);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 component needed as first parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, Class param1);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 String needed as first parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, String param1);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 component needed as first parameter to construct this component
     * @param param2 component needed as second parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, Class param1, Class param2);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 component needed as first parameter to construct this component
     * @param param2 String needed as second parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, Class param1, String param2);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 String needed as first parameter to construct this component
     * @param param2 component needed as second parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, String param1, Class param2);
    
    /**
     * Registers a component
     * 
     * @param key a key that identifies the component. Must be unique within the container.
     * @param implementation the concrete component class
     * @param param1 String needed as first parameter to construct this component
     * @param param2 String needed as second parameter to construct this component
     * 
     * @return the key used for registration
     */
    public Object reg(Object key, Class implementation, String param1, String param2);
    
    /**
     * Unregisters a component
     * 
     * @parameter key key of the component to unregister.
     * 
     * @return the unregistered component
     */
    public Object unreg(Object key);
}
